find_package(cx_freeze 5.0 REQUIRED)

configure_file(${CMAKE_CURRENT_LIST_DIR}/setup_win32.py.in setup.py @ONLY)
add_custom_target(build_bundle)
add_dependencies(packaging build_bundle)
add_dependencies(build_bundle projects)

add_custom_command(
    TARGET build_bundle PRE_LINK
    COMMAND ${CMAKE_COMMAND} -E remove_directory ${CMAKE_BINARY_DIR}/package
    COMMAND ${CMAKE_COMMAND} -E make_directory ${CMAKE_BINARY_DIR}/package
    COMMENT "Cleaning old package/ directory"
    WORKING_DIRECTORY ${CMAKE_BINARY_DIR}
)

add_custom_command(
    TARGET build_bundle POST_BUILD
    COMMAND ${PYTHON_EXECUTABLE} setup.py build_exe
    COMMENT "Running cx_Freeze"
    WORKING_DIRECTORY ${CMAKE_BINARY_DIR}
)

install(DIRECTORY ${CMAKE_BINARY_DIR}/package/
        DESTINATION "."
        USE_SOURCE_PERMISSIONS
        COMPONENT "_slicer" # Note: _ prefix is necessary to make sure the Cura component is always listed first
)

if(BUILD_OS_WIN32)
    install(FILES ${EXTERNALPROJECT_INSTALL_PREFIX}/vcredist_x32.exe
            DESTINATION "."
            COMPONENT "vcredist"
           )
else()
    install(FILES ${EXTERNALPROJECT_INSTALL_PREFIX}/vcredist_x64.exe
            DESTINATION "."
            COMPONENT "vcredist"
            )
endif()

include(CPackComponent)

cpack_add_component(_slicer DISPLAY_NAME "Cura Executable and Data Files" REQUIRED) 
cpack_add_component(vcredist DISPLAY_NAME "Install Visual Studio 2015 Redistributable")

set(CPACK_GENERATOR "NSIS")
set(CPACK_PACKAGE_NAME "3DGenceSlicer")
set(CPACK_PACKAGE_APP_NAME "3DGence Slicer")
set(CPACK_PACKAGE_VENDOR "3D Gence")
set(CPACK_PACKAGE_ICON_BMP "gence_small.bmp")
set(CPACK_PACKAGE_SPECIALBITMAP "gence.bmp")
set(CPACK_PACKAGE_SPECIALBITMAP_START "gence_start.bmp")
set(CPACK_PACKAGE_BASE_DIR "3DGence")
set(CPACK_PACKAGE_PRODUCT_NAME "Slicer")
set(CPACK_PACKAGE_LINK "3DGENCE SLICER (software, manual)")
set(CPACK_PACKAGE_LINK_LOCATION "https://3dgence.com/support/software.html")

set(CPACK_PACKAGE_VERSION_MAJOR ${CURA_VERSION_MAJOR})
set(CPACK_PACKAGE_VERSION_MINOR ${CURA_VERSION_MINOR})
set(CPACK_PACKAGE_VERSION_PATCH ${CURA_VERSION_PATCH})
set(CPACK_PACKAGE_VERSION ${CURA_VERSION})
set(CPACK_PACKAGE_SHORT_VERSION ${CURA_SHORT_VERSION})
set(CPACK_PACKAGE_INSTALL_FILE ${CMAKE_BINARY_DIR})
set(CPACK_RESOURCE_FILE_LICENSE ${CMAKE_SOURCE_DIR}/packaging/slicer_license)

set(CPACK_PACKAGE_FULL_NAME "3DGence Slicer ${CURA_VERSION_MAJOR}.${CURA_VERSION_MINOR}.${CURA_VERSION_PATCH}")
set(CPACK_PACKAGE_SHORT_NAME "3DGence Slicer ${CURA_VERSION_MAJOR}.${CURA_VERSION_MINOR}")

# CPackNSIS
set(CPACK_NSIS_INSTALLED_ICON_NAME "gence.ico")

# Needed to call the correct vcredist_x["32", "64"] executable
# TODO: Use a variable, which is already known. For example CPACK_SYSTEM_NAME -> "win32"
if(BUILD_OS_WIN32)
    set(CPACK_NSIS_PACKAGE_ARCHITECTURE "32")
else()
    set(CPACK_NSIS_PACKAGE_ARCHITECTURE "64")
endif()

set(CPACK_NSIS_PACKAGE_NAME ${CPACK_PACKAGE_NAME})

include(CPack)

add_custom_command(
    TARGET build_bundle POST_BUILD
    # NOTE: Needs testing here, whether CPACK_SYSTEM_NAME is working good for 64bit builds, too.
    COMMAND ${CMAKE_COMMAND} -E copy_directory ${CMAKE_SOURCE_DIR}/img ${CMAKE_BINARY_DIR}/_CPack_Packages/${CPACK_SYSTEM_NAME}/NSIS
    COMMENT "Copying NSIS graphics"
)

add_custom_command(
    TARGET build_bundle POST_BUILD
    # NOTE: Needs testing here, whether CPACK_SYSTEM_NAME is working good for 64bit builds, too.
    COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_SOURCE_DIR}/packaging/slicer_license ${CMAKE_BINARY_DIR}/_CPack_Packages/${CPACK_SYSTEM_NAME}/NSIS
    COMMENT "Copying slicer_license.txt"
)