# Signing the Installer and Cura.exe with "signtool.exe"
# Location on my desktop:
# -> C:\Program Files (x86)\Windows Kits\10\bin\x86\signtool.exe
# * Manual: https://msdn.microsoft.com/de-de/library/8s9b9yaz(v=vs.110).aspx

find_package(Signtool REQUIRED)

set(signtool_OPTIONS /n "3dgence" /t http://time.certum.pl /fd sha256 /v)
set(CPACK_PACKAGE_NAME "3DGenceSlicer")
set(signtool_OPTIONS ${signtool_OPTIONS})

message(STATUS "Signing executables : ")

# Signing 3DGence Slicer.exe
add_custom_command(
    TARGET signing PRE_BUILD
    COMMAND scsigntool sign ${signtool_OPTIONS} "3DGence Slicer.exe"
	COMMENT "Signing 3DGence Slicer.exe"
	## Other optional options:
	# /tr timestampServerUrl 
    WORKING_DIRECTORY ${CMAKE_BINARY_DIR}/package
)

# Signing CuraEngine.exe
add_custom_command(
	TARGET signing PRE_BUILD
	COMMAND scsigntool sign ${signtool_OPTIONS} CuraEngine.exe
	COMMENT "Signing CuraEngine.exe"
	## Other optional options:
	# /tr timestampServerUrl 
	WORKING_DIRECTORY ${CMAKE_BINARY_DIR}/package
)

# Signing the installer
add_custom_target(signing-installer) # Sadly "TARGET package POST_BUILD" can't be used in the following add_custom_command()
set(CURA_INSTALLER_NAME "${CPACK_PACKAGE_NAME}-${CURA_VERSION}-win64.exe")

add_custom_command(
	TARGET signing-installer
	COMMAND scsigntool sign ${signtool_OPTIONS} ${CURA_INSTALLER_NAME}
	## Other optional options:
	# /tr timestampServerUrl 
	WORKING_DIRECTORY ${CMAKE_BINARY_DIR}
)