from UM.Scene.Selection import Selection
from UM.Scene.SceneNode import SceneNode

from UM.Math.Vector import Vector
from UM.Math.Quaternion import Quaternion

from UM.Application import Application

from cura.SIProxies.CameraOperator import createDataCameraOperator
from cura.SIProxies.CameraOperator import CameraOperator
from cura.SIProxies.SettingsManage.PrinterSettingsDB import PrinterSettingsDB

import math

class ReversedPlateManager:
    def __init__(self):
        self._printerSettingsDB = PrinterSettingsDB()

    _instance = None

    @classmethod
    def getInstance(cls):
        if ReversedPlateManager._instance is None:
            ReversedPlateManager._instance = cls()

        return ReversedPlateManager._instance

    def reversePlateIfNecessary(self):
        if self._printerSettingsDB.is_printer_mirrored():
            self._rotateAllModels()
            self._flipCamera()
            self._flipFrontIndicator()
            self._flipCoordinateSystem()

    def _rotateAllModels(self):
        for node in Selection().getAllSelectableNodes():
            current_position = node.getPosition()
            rotation = Quaternion.fromAngleAxis(math.pi, Vector.Unit_Y)
            node.rotate(rotation, SceneNode.TransformSpace.World)
            node.setPosition(Vector(-current_position.x, current_position.y, -current_position.z))

    def _flipFrontIndicator(self):
        buildVolume = Application.getInstance().getBuildVolume()
        buildVolume.reversePlate()

    def _flipCamera(self):
        cameraOperator = createDataCameraOperator()
        cameraOperator.reverseCamera()

    def _flipCoordinateSystem(self):
        coordinateSystem = Application.getInstance()._coordinateSystem
        coordinateSystem.reverse()