import math
import itertools
import numpy
from PIL import Image
from cura.SIProxies.AutoarrangePlugin.OccupiedChunk import OccupiedChunk

def chunk_model(bitmap, px_chunk_size):
    x_chunks = math.ceil(bitmap.width/px_chunk_size)
    y_chunks = math.ceil(bitmap.height/px_chunk_size)
    aligned_bitmap_width = x_chunks*px_chunk_size
    aligned_bitmap_height = y_chunks*px_chunk_size
    aligned_bitmap_size = (aligned_bitmap_width, aligned_bitmap_height)

    aligned_bitmap = Image.new("1", aligned_bitmap_size, 1)

    aligned_position = (math.floor((aligned_bitmap.width-bitmap.width)/2),
                        math.floor((aligned_bitmap.height-bitmap.height)/2))

    aligned_bitmap.paste(bitmap, aligned_position)

    # aligned_bitmap.save("aligned.bmp")

    chunked_bitmap_array = numpy.zeros((y_chunks, x_chunks), dtype=bool)

    iterator = numpy.nditer(chunked_bitmap_array, flags=['multi_index'], op_flags=['readwrite'])
    for chunk in iterator:
        for x_coordinate, y_coordinate in itertools.product(range(px_chunk_size), repeat=2):
            bitmap_coordinates = (px_chunk_size*iterator.multi_index[1] + y_coordinate,
                                  px_chunk_size*iterator.multi_index[0] + x_coordinate)
            pixel_color = aligned_bitmap.getpixel(bitmap_coordinates)
            if pixel_color == 255:
                chunk[...] = True
                break

    # normalized_array = chunked_bitmap_array * 255
    # array_to_save = numpy.uint8(normalized_array)
    # chunked_to_save = Image.fromarray(array_to_save)
    # chunked_to_save.save("chunked.png")

    return chunked_bitmap_array

def check_overlapping(vector1, vector2, enlarge_factor):
    result = False

    enlarged_vector = OccupiedChunk(vector1.x_min - enlarge_factor, vector1.x_max + enlarge_factor)

    if enlarged_vector < vector2:
        if enlarged_vector.x_max >= vector2.x_min:
            result = True
    else:
        if vector2.x_max >= enlarged_vector.x_min:
            result = True

    return result