# Copyright (c) 2017 vabi vabi vabi
# this->Class is released under the terms of the AGPLv3 or higher.

from UM.Mesh.MeshBuilder import MeshBuilder
from UM.Math.Vector import Vector
from UM.Math.Color import Color
from UM.Scene.SceneNode import SceneNode
from UM.Scene.Camera import Camera

from UM.View.GL.OpenGL import OpenGL
from UM.Resources import Resources

from UM import Application


import numpy
import numpy.linalg
import copy


class CoordinateSystem(SceneNode):
    dimmer = 0.6
    RedColor = Color(int(215 * dimmer), int(12 * dimmer),
                     int(31 * dimmer), 255)
    BlueColor = Color(int(41 * dimmer), int(186 * dimmer),
                      int(10 * dimmer), 255)
    GreenColor = Color(int(8 * dimmer), int(140 * dimmer),
                       int(121 * dimmer), 255)

    def __init__(self, camera):
        super().__init__(camera)
        self._camera = camera
        self._origin_mesh = None
        self._lastState = None
        self._shader = None
        self._reversed = False
        self._calculate_aabb = False

        self.buildCoordinateSystem()


    def buildCoordinateSystem(self):
        mesh_builder = MeshBuilder()
        self.system_size = 0.08
        shift = self.system_size / 2
        axis_thickness = self.system_size / 30
        dart_size = self.system_size / 10

        self._lastState = self._camera.isPerspective()

        # z-Axis
        mesh_builder.addCube(
            width=axis_thickness,
            height=self.system_size,
            depth=axis_thickness,
            center=Vector(0, shift, 0),
            color=CoordinateSystem.GreenColor
        )
        mesh_builder.addPyramid(
            width=dart_size,
            height=dart_size,
            depth=dart_size,
            center=Vector(0, self.system_size, 0),
            color=CoordinateSystem.GreenColor
        )

        # y-axis
        mesh_builder.addCube(
            width=axis_thickness,
            height=axis_thickness,
            depth=self.system_size,
            center=Vector(0, 0, shift) if self._reversed else Vector(0, 0, -shift),
            color=CoordinateSystem.BlueColor
        )
        mesh_builder.addPyramid(
            width=dart_size,
            height=dart_size,
            depth=dart_size,
            center=Vector(0, 0, self.system_size) if self._reversed else Vector(0, 0, -self.system_size),
            angle=-90 if self._reversed else 90,
            axis=Vector(1, 0, 0),
            color=CoordinateSystem.BlueColor
        )

        # x-Axis
        mesh_builder.addCube(
            width=self.system_size,
            height=axis_thickness,
            depth=axis_thickness,
            center=Vector(-shift, 0, 0) if self._reversed else Vector(shift, 0, 0),
            color=CoordinateSystem.RedColor
        )
        mesh_builder.addPyramid(
            width=dart_size,
            height=dart_size,
            depth=dart_size,
            center=Vector(-self.system_size, 0, 0) if self._reversed else Vector(self.system_size, 0, 0),
            angle=-90 if self._reversed else 90,
            axis=Vector(0, 0, 1),
            color=CoordinateSystem.RedColor
        )

        self._origin_mesh = mesh_builder.build()
        self.setMeshData(self._origin_mesh)

        self.setPosition(Vector(0.16, -0.12, -0.12))

    def _setSystemSize(self, size):
        scale = size / self.system_size
        self.setScale(Vector(scale, scale, scale))

    def render(self, renderer):
        if not self.getMeshData():
            return True

        # self._shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "color.shader"))
        if not self._shader:
            self._shader = OpenGL.getInstance().createShaderProgram(
                Resources.getPath(Resources.Shaders, "hud.shader"))

        # renderer.queueNode(self, mode = RenderBatch.RenderMode.Lines)
        # self.setPosition(Vector(0,0,0))

        windowSize = Vector(self._camera.getViewportWidth(),
                            self._camera.getViewportHeight(), 0.0)
        renderer.queueNode(self, mesh=self.getMeshData(), shader=self._shader,
                           withoutcamera=True, uniforms={"windowsize": windowSize}, overlay=True)

        return True

    #  Used by ReversePlateManager to flip coordinate system 
    def reverse(self):
        self._reversed = not self._reversed
        self.buildCoordinateSystem()
