import os

# cura.Settings imports
from cura.Settings.ContainerManager import ContainerManager
from cura.SIProxies.SettingsManage.ProfileLoaderManager import ProfileLoaderManager

# UM.Settings imports
from UM.Settings.ContainerRegistry import ContainerRegistry
from UM.Settings.Models.DefinitionContainersModel import DefinitionContainersModel
from UM.Settings.Models.ContainerStacksModel import ContainerStacksModel

# UM.Preferences import
from UM.Resources import Resources

# UM.Applications
from UM.Application import Application

from .SettingsManage.PrinterSettingsDB import PrinterSettingsDB

class GencePrinterManager():

    ##  Get the singleton instance for this class.
    @classmethod
    def getInstance(cls):
        if GencePrinterManager.__instance is None:
            currentlyUsedMachineManager = Application.getInstance().getMachineManager()
            GencePrinterManager.__instance = cls(currentlyUsedMachineManager)
        return GencePrinterManager.__instance

    __instance = None

    def __init__(self, machineManager):
        self._machineManager = machineManager
        self._currentPrinter = machineManager.activeMachineName
        # self._currentPrinterPath = None

        self._loadPrinters()
        self._database = PrinterSettingsDB()
        self._lastLoadedProfile = None

        # setActivePrinter from saved
        # Preferences.getInstance().addPreference("cura/active_profile", "")
        # active_profile_path = Preferences.getInstance().getValue("cura/active_profile")

        # if active_profile_path != "":
        #     # An active profile path was saved, so restore it.
        #     self.changePrinterSettings(active_machine_path)

    def changePrinterSettings(self, new_profile):
        # get sixth argument from the end it's printer name
        printer_name = new_profile['printer']
        nozzle = new_profile['nozzle']
        material = new_profile['material']
        quality = new_profile['quality']
        durability = new_profile['durability']

        profile = self._database.get_profile(printer_name, nozzle, material, quality, durability)

        if len(profile) == 0:
            return False

        printer_name = profile[0][0]

        if printer_name != self._currentPrinter:
            self.changePrinter(printer_name)

        currentQuality = self._machineManager.activeQualityContainerId

        if currentQuality != "normal":
            self._machineManager.setActiveQuality("normal")

        ContainerManager.getInstance().removeQualityChanges("*")

        if profile[0][1] is not None:
            temp_file = os.path.join(Resources.getPath(Resources.GenceSettings), "current.profile")

            with open(temp_file, 'w+b') as f:
                f.write(profile[0][1])

            ContainerManager.getInstance().importProfileFromPlainPath(temp_file)

        profiles = ContainerRegistry.getInstance().findInstanceContainers(name = "current")

        if profiles:
            settings_id = profiles[0].id
            result = self._machineManager.setActiveQuality(settings_id)

            self.setProfileLoadedStatus(result)

            if result == True:
                self._lastLoadedProfile = new_profile

            return result
        
        self.setProfileLoadedStatus(False)
        return False

        # Preferences.getInstance().setValue("cura/active_profile", path)

    def changePrinter(self, printer, saveLastLoaded = True):
        result = self._machineManager.setActiveMachine(printer)
        if result and saveLastLoaded:
            self._currentPrinter = printer

        return result

    def setProfileFromPath(self, path, profileName):
        # Just to revert after all profile switching
        profileUsedBefore = self._lastLoadedProfile
        currentQuality = self._machineManager.activeQualityContainerId

        if currentQuality != "normal":
            self._machineManager.setActiveQuality("normal")
        
        result = ContainerManager.getInstance().importProfileFromPlainPath(path)

        if result["status"] == "ok":
            profiles = ContainerRegistry.getInstance().findInstanceContainers(name = profileName)
            machine_name = profiles[0].getProperty('machine_name', 'value')
            ProfileLoaderManager.GetInstance().set_printer_by_real_name(machine_name)
            if machine_name != self._currentPrinter:
                return False

            ContainerManager.getInstance().removeQualityChanges("*")
            ContainerManager.getInstance().importProfileFromPlainPath(path)

        profiles = ContainerRegistry.getInstance().findInstanceContainers(name = profileName)

        self._lastLoadedProfile = profileUsedBefore

        if profiles:
            settings_id = profiles[0].id
            result = self._machineManager.setActiveQuality(settings_id)

            self.setProfileLoadedStatus(result)

            Application.getInstance().sendCustomProfileLoaded()

            return result

        self.setProfileLoadedStatus(False)
        return False

    def reloadProfile(self):
        #self.changePrinterSettings(self._lastLoadedProfile)
        ProfileLoaderManager.GetInstance().force_set_profile(self._lastLoadedProfile)

    def _loadPrinters(self):
        existing_definitions = DefinitionContainersModel()._items

        # Remove all existing machines, just to avoid doubling stacks
        existing_stacks = ContainerStacksModel()._items
        for stack in existing_stacks:
            if stack['metadata']['type'] == "machine":
                self._machineManager.removeMachine(stack['id'])
        
        for definition in existing_definitions:
            if definition['metadata']['type'] == "machine" and definition['id'] != "fdmprinter":
                self._machineManager.addMachine(definition['name'], definition['id'])
        
        return None

    def setProfileLoadedStatus(self, result: bool):
        genceApp = Application.getInstance()
        if result == True:
            genceApp.displayStatus("Profile loaded.")
        else:
            genceApp.displayStatus("ERROR: Profile not loaded.")