
from PyQt5.QtCore import Qt, QCoreApplication, pyqtSlot, pyqtProperty
from UM.Application import Application

from UM.Qt.ListModel import ListModel
from .NozzlesModel import NozzlesModel
from .MaterialsModel import MaterialsModel
from .QualitiesModel import QualitiesModel


class DurabilitiesModel(ListModel):
    QualityNameRole = Qt.UserRole + 1

    def __init__(self, parent=None):

        if Application.getInstance()._durabilities != None:
            raise ValueError(
                "Duplicate singleton creation of DurabilitiesModel")
        Application.getInstance()._durabilities = self

        super().__init__(parent)
        self._printerTree = Application.getInstance()._printerTree
        self.addRoleName(self.QualityNameRole, "name")
        self._update()

    def _update(self):
        items = []
        quality = DurabilitiesModel.getInstance().__Quality

        if(not len(quality) == 0):
            printer = NozzlesModel.getInstance().getPrinter()
            nozzle = MaterialsModel.getInstance().getNozzle()
            material = QualitiesModel.getInstance().getMaterial()

            node = self._printerTree.getDurabilitiesNode(
                printer, nozzle, material, quality)
            nozzles = node.getCurrentNode()[2]

            for i in nozzles:
                item = {"name": i}
                items.append(item)

        self.setItems(items)

    def getMaterial(self):
        return DurabilitiesModel.getInstance().__Quality

    __Quality = ""

    @classmethod
    def getInstance(cls):
        if Application.getInstance()._durabilities is None:
            Application.getInstance()._durabilities = cls()

        return Application.getInstance()._durabilities

    @staticmethod
    def onQualityChanged(newQuality):

        DurabilitiesModel.getInstance().__Quality = newQuality
        DurabilitiesModel.getInstance()._update()


def CreateDurabilitiesModel(engine, script_engine):
    return DurabilitiesModel.getInstance()
