import os

from PyQt5.QtCore import QObject, pyqtSignal

from UM.Resources import Resources

from . import TreeNode


class PrinterTree(QObject):

    __instance = None
    
    @classmethod
    # Get the singleton instance for this class.
    def getInstance(cls):
        if PrinterTree.__instance is None:
            PrinterTree.__instance = cls()
        return PrinterTree.__instance


    def __init__(self):
        super().__init__(None)
        self._tree = TreeNode("Empty")
        self._customProfileLoaded = False

    _instance = None
    _lastSelection = None

    selectedPrinterChanged = pyqtSignal(dict)

    def setRoot(self, tree: TreeNode):
        self._tree = tree

    def getPrintersNode(self)->TreeNode:
        return self._tree

    def getNozzlesNode(self, printerName: str=None)->TreeNode:
        return self._tree.getNextNode(printerName)

    def getMaterialsNode(self, printerName: str, nozzleSize: str):
        return self._tree.getNextNode(printerName).getNextNode(nozzleSize)

    def getQualitiesNode(self, printerName: str, nozzleSize: str, material: str):
        return self.getMaterialsNode(printerName, nozzleSize).getNextNode(material)

    def getDurabilitiesNode(self, printerName: str, nozzleSize: str, material: str, quality: str):
        return self.getQualitiesNode(printerName, nozzleSize, material).getNextNode(quality)

    def onDurabilitySelected(self, durability: str, quality: str, material: str, nozzle: str, printer: str):
        if printer and nozzle and material and quality and durability:
            profile_info = {
                'printer': printer,
                'nozzle': nozzle,
                'material': material,
                'quality': quality,
                'durability': durability
            }

            if self._customProfileLoaded or self.__isNewProfile(profile_info):
                self._customProfileLoaded = False
                PrinterTree._lastSelection = profile_info
                self.selectedPrinterChanged.emit(PrinterTree._lastSelection)

    def __isNewProfile(self, new_profile):
        if PrinterTree._lastSelection is None:
            return True

        shared_items = set(PrinterTree._lastSelection.items()) & set(new_profile.items())

        if len(shared_items) == len(new_profile):
            return False
        return True
