from UM.View.RenderPass import RenderPass
from UM.Resources import Resources
from UM.Application import Application

from UM.Math.Color import Color

from UM.Scene.Selection import Selection
from UM.Scene.ToolHandle import ToolHandle
from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator

from UM.View.RenderPass import RenderPass
from UM.View.RenderBatch import RenderBatch
from UM.View.GL.OpenGL import OpenGL
from UM.Scene.Camera import Camera



class ShadersProxy(RenderPass):
    def __init__(self, width, height):
        super().__init__("shadersproxy", width, height, -999)
        self._shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "empty.shader"))
        self._gl = OpenGL.getInstance().getBindingsObject()
        self._scene = Application.getInstance().getController().getScene()
        self._renderer = Application.getInstance().getRenderer()

        self._selection_map = {}
        self._output = None

    def render(self):

        batch = RenderBatch(self._shader)


        newCam = Camera("second",self._scene.getRoot())
        newCam.setEnabled(True)
        newCam.setPerspective(True)


        selectable_objects = False
        for node in DepthFirstIterator(self._scene.getRoot()):
            if (not node.isSelectable()) and node.getMeshData() and hasattr(node, '_grid_mesh'):

                selectable_objects = True
                uniform = {"u_fieldSize" : [node._width,node._depth]}
                newCam.setPosition(self._scene.getActiveCamera().getPosition())
                newCam.setOrientation(self._scene.getActiveCamera().getOrientation())
                newCam.setViewportSize(self._scene.getActiveCamera().getViewportWidth(),
                                       self._scene.getActiveCamera().getViewportHeight())



                batch.addItem(transformation=node.getWorldTransformation(), mesh=node._grid_mesh, uniforms=uniform)

        self.bind()
        if selectable_objects:


            batch.render(self._scene.getActiveCamera())

            self._gl.glColorMask(self._gl.GL_TRUE, self._gl.GL_TRUE, self._gl.GL_TRUE, self._gl.GL_FALSE)
            self._gl.glDisable(self._gl.GL_DEPTH_TEST)

        # output = self.getOutput()
        # if (not output == None):
        #     output.save("/Users/kordiaczynski/Desktop/selection/sel.png");

        self.release()


