# @author Tomasz Drosik (t.drosik@profisatwork.pl)
# @date   03.07.17
#
# simple combo box manager
#
# observes application screen and manages closing of combo boxes
# it's a kind of model object for the RetractableList.qml component
# [Cura/resources/qml/Gence/list/]
#
# TODO: implement View hierarchy if necessary
from PyQt5.QtCore import pyqtProperty, QObject, pyqtSignal, pyqtSlot


class ComboBoxManager(QObject):

    # called on request for close
    hideComboBoxes = pyqtSignal(int, int, arguments=['mx', 'my'])

    # called on turning on combo box
    comboBoxEnabled = pyqtSignal()

    # forces hiding - called by 'awareness area'
    # @param x mouse reaction position
    # @param y mouse reaction position
    @pyqtSlot(int, int)
    def forceHide(self, x, y):
        self.hideComboBoxes.emit(x, y)

    # informs about showing combo box
    @pyqtSlot()
    def shown(self):
        self.comboBoxEnabled.emit()

    # initializes mechanism
    def initialize(self, engine):
        # registers class on a QML side
        engine.rootContext().setContextProperty("combo_box_manager", self)

    # c-tor
    def __init__(self):
        """ call QObject c-tor """
        super().__init__(None)
