from PyQt5.QtCore import pyqtProperty, QObject, pyqtSignal
from UM.Application import Application

class RightMenuManager(QObject):
    _instance = None

    changePrinterMenu = pyqtSignal(str, str, arguments=['value', "active"])
    changeNozzleMenu = pyqtSignal(str, str, arguments=['value', "active"])
    changeMaterialMenu = pyqtSignal(str, str, arguments=['value', "active"])
    changeQualityMenu = pyqtSignal(str, str, arguments=['value', "active"])
    changeDurabilityMenu = pyqtSignal(str, str, arguments=['value', "active"])

    def __init__(self):
        """ call QObject c-tor """
        super().__init__(None)

    def initialize(self, engine):
        engine.rootContext().setContextProperty("right_menu_manager", self)

    @classmethod
    def GetInstance(cls):
        if RightMenuManager._instance is None:
            RightMenuManager._instance = cls()

        return RightMenuManager._instance

    def setPrinterMenu(self, value: str, active: bool = None):
        self.changePrinterMenu.emit(value, self._parse_active(active))

    def setNozzleMenu(self, value: str, active: bool = None):
        self.changeNozzleMenu.emit(value, self._parse_active(active))

    def setMaterialMenu(self, value: str, active: bool = None):
        self.changeMaterialMenu.emit(value, self._parse_active(active))

    def setQualityMenu(self, value: str, active: bool = None):
        self.changeQualityMenu.emit(value, self._parse_active(active))

    def setDurabilityMenu(self, value: str, active: bool = None):
        self.changeDurabilityMenu.emit(value, self._parse_active(active))

    def set_profiles_ui(self, profile):
        if profile[0]: self.setPrinterMenu(profile[0][0], profile[0][1])
        if profile[1]: self.setNozzleMenu(profile[1][0], profile[1][1])
        if profile[2]: self.setMaterialMenu(profile[2][0], profile[2][1])
        if profile[3]: self.setQualityMenu(profile[3][0], profile[3][1])
        if profile[4]: self.setDurabilityMenu(profile[4][0], profile[4][1])

    def _parse_active(self, active: bool):
        if active is None:
            return "none"
        if active is False:
            return "false"
        if active is True:
            return "true"
        return "true"


