# Copyright (c) 2015 Ultimaker B.V.
# Uranium is released under the terms of the AGPLv3 or higher.

from UM.Scene.SceneNode import SceneNode
from UM.Math.Vector import Vector
from UM.Operations.RotateOperation import RotateOperation

from UM.Operations.Operation import Operation

##  Operation that rotates a scene node.
class GenceRotateOperation(RotateOperation):
    def __init__(self, node, rotation, old_rotation, emiter, rotate_around_point=Vector(0, 0, 0), mergeKey=None):
        super().__init__(node,rotation,rotate_around_point)
        self._mergeKey= mergeKey
        self._always_merge=True
        self._old_rotation = old_rotation
        self._node_rotation = (node._rotationX, node._rotationY, node._rotationZ)

        self._emiter = emiter

    ##  Undoes the rotation, rotating the node back.
    def undo(self):
        setattr(self._node, "_rotationX", self._old_rotation[0])
        setattr(self._node, "_rotationY", self._old_rotation[1])
        setattr(self._node, "_rotationZ", self._old_rotation[2])

        self._emiter.emit()
        super().undo()


    ##  Redoes the rotation, rotating the node again.
    def redo(self):
        setattr(self._node, "_rotationX", self._node_rotation[0])
        setattr(self._node, "_rotationY", self._node_rotation[1])
        setattr(self._node, "_rotationZ", self._node_rotation[2])

        self._emiter.emit()
        super().redo()

    def mergeWith(self, other):
            if type(other) is not GenceRotateOperation:
                return False
            if other._node != self._node:  # Must be operations on the same node.
                return False
            if other._mergeKey != self._mergeKey:
                return False

            op = GenceRotateOperation(self._node, other._rotation * self._rotation, other._old_rotation,self._emiter,self._rotate_around_point,self._mergeKey)
            op._old_transformation = other._old_transformation  # Use the _old_transformation of the oldest rotation in the new operation.
            return op