pragma Singleton

import QtQuick 2.2
import QtQuick.Controls 1.1

import SI 1.0 as SI
import UM 1.1 as UM
import Gence 1.0 as Gence
import Settings 1.0 as Settings

Item
{
    //Right menu actions
    property alias onPrinterChanged: onPrinterChanged
    property alias onNozzleChanged: onNozzleChanged

    // used by normal and abnormal undo to force enabling abnormal undo (used by CuraEngineBackend when slicing request was not successful).
    property bool abnormalUndoEnabled: false

    Connections
    {
        target: CuraApplication
        onAbnormalUndoRequest:
        {
            abnormalUndoEnabled = true
            console.log("AbnormalUndoTriggered")
            abnormalUndo.trigger()
            abnormalUndoEnabled = false
        }
    }


    property int numberOfSelectedFiles: 0

    Component.onCompleted:
    {
        Gence.Notifications.setFunction("onEmptySelection",selectionEmpty.trigger)
        Gence.Notifications.setFunction("onNonEmptySelection",onSelection.trigger)
        Gence.Notifications.setFunction("onMultipleSelection", multipleModelsSelected.trigger)
        Gence.Notifications.setFunction("onShowTranslateView",showMoveTool.trigger)
        Gence.Notifications.setFunction("onShowRotateView",showRotateTool.trigger)
        Gence.Notifications.setFunction("onShowScaleView",showScaleTool.trigger)
        Gence.Notifications.setFunction("onShowSplitTool",showSplitTool.trigger)
        Gence.Notifications.setFunction("onNodeTransformed",modelWasMoved.trigger)
        Gence.Notifications.setFunction("onSliced",showSliced.trigger)
        Gence.Notifications.setFunction("onAppStateChanged",onAppStateChanged)
        Gence.Notifications.setFunction("onLayerWasChanged",layerWasChanged.trigger)
        Gence.Notifications.setFunction("onUpdateSplitTool",splitToolLimitsChanged.trigger)
    }

    property alias empty_plate:empty_plate
    property alias node_loading:node_loading
    property alias node_loaded:node_loaded
    property alias slicing:slicing
    property alias deactivate_right_menu:deactivate_right_menu
    property alias activate_right_menu:activate_right_menu
    property alias split_deactivate_left_menu:split_deactivate_left_menu
    property alias split_succ_activate_left_menu:split_succ_activate_left_menu
    property alias split_fail_activate_left_menu:split_fail_activate_left_menu
    property alias gcode_loaded:gcode_loaded
    property alias gcode_loading: gcode_loading
    property alias sliced:sliced
    property alias abnormalUndo:abnormalUndo
    property alias save:saved
    property alias error_disconnected:error_disconnected
    property alias error_unabletoread:error_unabletoread
    property alias error_unabletosave:error_unabletosave
    property alias undo:undo
    property alias modalWindowOpened: modalWindowOpened
    property alias modalWindowClosed: modalWindowClosed

    Action
    {
        id:splitToolLimitsChanged
    }
    Action
    {
        id:layerWasChanged
    }

    Action
    {
        id:empty_plate
    }
    Action
    {
        id:node_loading
    }
    Action
    {
        id:node_loaded
    }
    Action
    {
        id:gcode_loading
    }
    Action
    {
        id:gcode_loaded
    }
    Action
    {
        id:slicing
    }
    Action
    {
        id:deactivate_right_menu
    }
    Action
    {
        id:activate_right_menu
    }
    Action
    {
        id:split_deactivate_left_menu
    }
    Action
    {
        id:split_succ_activate_left_menu
    }
    Action
    {
        id:split_fail_activate_left_menu
    }


    Action
    {
        id:sliced
    }
    Action
    {
        id:saved
    }
    Action
    {
        id:error_disconnected
    }
    Action
    {
        id:error_unabletosave
    }
    Action
    {
        id:error_unabletoread
    }
    Action
    {
        id: modalWindowOpened
    }
    Action
    {
        id: modalWindowClosed
    }

    function onAppStateChanged(appState)
    {
       switch(appState)
       {
                case Gence.Notifications.EMPTY_PLATE:
                    empty_plate.trigger()
                    break;
                case Gence.Notifications.NODE_LOADING:
                    node_loading.trigger()
                    break;
                case Gence.Notifications.NODE_LOADED:
                    node_loaded.trigger()
                    break;
                case Gence.Notifications.GCODE_LOADING:
                    gcode_loading.trigger()
                    break;
                case Gence.Notifications.GCODE_LOADED:
                    gcode_loaded.trigger()
                    break;
                case Gence.Notifications.SLICING:
                    slicing.trigger()
                    break;
                case Gence.Notifications.SLICED:
                    sliced.trigger()
                    break;
                case Gence.Notifications.SAVED:
                    saved.trigger()
                    break;
                case Gence.Notifications.ERROR_DISCONNECTED :
                    error_disconnected.trigger()
                    break;
                case Gence.Notifications.ERROR_UNABLETOSAVE:
                    error_unabletosave.trigger()
                    break;
                case Gence.Notifications.ERROR_UNABLETOREAD :
                    error_unabletoread.trigger()
       };
    }

    property alias layerWasChanged: layerWasChanged
    property alias multipleModelsSelected: multipleModelsSelected
    property alias modelWasMoved: modelWasMoved


    property alias showRotateTool:showRotateTool
    property alias showMoveTool:showMoveTool
    property alias showScaleTool:showScaleTool
    property alias showSplitTool:showSplitTool

    property alias showSliced:showSliced
    property alias hideAllTools: hideAllTools
    property alias activeAllTools:activeAllTools
    property alias disableAllTools:disableAllTools
    property alias duplicateElement: duplicateElement
    property alias selectionEmpty:onSelectionEmpty
    property alias selection: onSelection
    property alias splitToolLimitsChanged: splitToolLimitsChanged


    property alias orderTranslateTool: orderTranslateTool
    property alias orderRotateTool:orderRotateTool
    property alias orderScaleTool:orderScaleTool
    property alias orderSlicing:orderSlicing
    property alias orderMeshSplitterTool:orderMeshSplitterTool
    property alias arrange:arrange

    Action
    {
        id:arrange
        onTriggered: Gence.Autoarrange.arrange()
    }

    Action
    {
        id:duplicateElement
        onTriggered: Printer.duplicateNode();
    }

    Action
    {
        id:orderRotateTool
        onTriggered: UM.Controller.setActiveTool("GenceRotateTool")
    }

    Action
    {
        id:orderScaleTool
        onTriggered: UM.Controller.setActiveTool("GenceScaleTool")
    }
    Action
    {
        id:orderTranslateTool
        onTriggered: UM.Controller.setActiveTool("SITranslateTool")
    }

    Action
    {
        id:orderSlicing
        onTriggered:Gence.Notifications.showSliced()

    }

    Action
    {
        id:orderMeshSplitterTool
        onTriggered: UM.Controller.setActiveTool("GenceSplitterTool")
    }

    //Called from python

    // Called when nodes were sliced
    Action
    {
        id:showSliced
        onTriggered: disableAllTools.trigger()
    }

    // Called when nothing is selected
    Action
    {
        id:onSelectionEmpty
        onTriggered: disableAllTools.trigger()
    }

    //Called when something was selected
    Action
    {
        id:onSelection
        onTriggered:
        {
            activeAllTools.trigger()
        }
    }

    //Called when there was some transformation performed selected node ( By Tool)
    Action
    {
        id: modelWasMoved

    }

    //Called when RotateTool view is required
    Action
    {
        id:showRotateTool
        enabled: UM.Controller.toolsEnabled
    }

    //Called when TranslateTool view is required
    Action
    {
        id:showMoveTool
        enabled:UM.Controller.toolsEnabled
    }

    //Called when ScaleTool view is required
    Action
    {
        id:showScaleTool
        enabled:UM.Controller.toolsEnabled
    }

    Action
    {
        id: showSplitTool
        enabled: UM.Controller.toolsEnabled
    }

    //Called when all tools should be close
    Action
    {
        id:hideAllTools
    }

    //Active all tools , make them clickable
    Action
    {
        id:activeAllTools
        enabled:UM.Controller.toolsEnabled
    }
    Action
    {
        id:disableAllTools
    }

    Action
    {
        id: multipleModelsSelected
    }

    //Normal undo, triggered when we are in SolidView and there are still some operations in OperationsStack
    Action
    {
        id:normalUndo
        enabled: Gence.Notifications.appState === Gence.Notifications.NODE_LOADED && !abnormalUndo.enabled && !abnormalUndoEnabled
        onTriggered:
        {
            console.log("Normal undo")
            UM.OperationStack.undo()

        }
    }

    //Abnormal undo, triggered when we are in layer view and we want to go back
    Action
    {
        id:abnormalUndo
        enabled: (abnormalUndoEnabled || Gence.Notifications.appState === Gence.Notifications.SLICED || Gence.Notifications.appState === Gence.Notifications.GCODE_LOADED) && !normalUndo.enabled
        onTriggered:
        {
            console.log("Abnormal undo")

            if(Settings.Notifications.is_xy_compensation){
                UM.OperationStack.undo()
            }

            if(Gence.Notifications.appState === Gence.Notifications.GCODE_LOADED){
                Gence.Notifications.reversePlateIfNecessary()
                UM.OperationStack.undo()
                Settings.Notifications.unset_gcode_profile()
                Gence.Notifications.showSolid(false)
            } else{
                Gence.Notifications.showSolid()
            }

        }
    }
    Action
    {
        id:undo
        onTriggered:
        {
            normalUndo.trigger()
            abnormalUndo.trigger()
        }
    }

    //Rigth Menu actions
    Action
    {
        id:onPrinterChanged
    }
    Action
    {
        id:onNozzleChanged
    }
}
