import QtQuick 2.4
import QtQuick.Layouts 1.1
import UM 1.3 as UM


import "../singletons"
import "../tweaks"

Item {
    id: button

    property bool cornerTriangleExists: true
    property bool cornerTriangleInColor: true
    property alias buttonText: buttonText.text
    property alias buttonTextVisibility: buttonText.visible
    property alias buttonIcon: buttonIcon.source
    property alias buttonIconWhite: buttonIconWhite.source
    property alias buttonAction: mouseArea
    property alias buttonTextBackgroundHeight: textBackground.height
    property alias buttonWidth: button.width

    property alias buttonClicked: buttonIconWhite.visible

    property alias buttonBackgroundColor: buttonBackground.color

    property alias deactivateButton: deactivateIndicator.visible

    property alias helpText: buttonTweaks.helpText
    property alias helpImageSource: buttonTweaks.helpImageSource
    property bool helpEnabled: true

    Layout.preferredHeight: Globals.buttonHeight
    Layout.fillWidth: true

    // Connections
    // {
    //     target:mouseArea
    //     onClicked:UM.Selection.isDeaf = false;
    // }

    MouseArea {
        id: mouseArea
        visible: !deactivateButton
        anchors.fill: parent
    }

    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: 0

        Rectangle {
            id: textBackground
            Layout.preferredHeight: Globals.buttonHeaderHeight
            Layout.fillWidth: true
            color: Colors.buttonTextBackground

            Text {
                id: buttonText
                anchors.centerIn: parent
                font.pixelSize: Globals.fontPixelSize
                font.capitalization: Font.AllUppercase
                font.family: Fonts.boxed

                color: Colors.buttonText
                text: "button"
            }
        }

        Rectangle {
            id: decorator
            Layout.fillWidth: true
            Layout.preferredHeight: Globals.decoratorHeight
            color: Colors.buttonDecorator
        }

        Rectangle {
            id: buttonBackground
            Layout.fillHeight: true
            Layout.fillWidth: true
            color: Colors.buttonBackground

            //button icon grey
            Image {
                id: buttonIcon
                visible: !(buttonIconWhite.visible)
                anchors.centerIn: parent
                height: Globals.leftMenuButtonIconSize
                mipmap: true
                fillMode: Image.PreserveAspectFit

                source: "../images/placeholder.svg"
            }

            //button icon white
            Image {
                id: buttonIconWhite
                visible: false
                anchors.centerIn: parent
                height: Globals.leftMenuButtonIconSize
                mipmap: true

                fillMode: Image.PreserveAspectFit

                source: "../images/placeholder_white.svg"
            }

            //Corner triangle
            Image {
                id: corner
                visible: cornerTriangleExists & cornerTriangleInColor
                height: Globals.buttonTriangleSize
                y: (parent.height * 0.93) - corner.height
                x: parent.width - (parent.height - y)

                fillMode: Image.PreserveAspectFit
                source: "../images/corner.svg"
            }

            //Corner triangle white
            Image {
                id: cornerWhite
                visible: !(corner.visible)
                y: (parent.height * 0.93) - corner.height
                x: parent.width - (parent.height - y)
                height: Globals.buttonTriangleSize

                fillMode: Image.PreserveAspectFit
                source: "../images/corner_white.svg"
            }
        }
    }

    //Non-active button layer
    Rectangle {
        id: deactivateIndicator
        anchors.fill: parent
        visible: false
        color: Colors.buttonDeactivator
    }

    BottomShadow {}

    ButtonTweaks {
        id: buttonTweaks
        hoverIndicatorEnabled: !deactivateButton && !buttonClicked
        helpEnabled: parent.helpEnabled ? !deactivateButton: false
    }
}
