import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 2.0
import QtQuick.Dialogs 1.2

import Gence 1.0 as Gence
import Cura 1.0 as Cura
import UM 1.3 as UM

import "../../tweaks"
import "../../buttons"
import "../../singletons"

Item {
    property alias layoutHeight: layout.height

    ColumnLayout {
        id: layout
        spacing: parent.width * 0.04
        anchors.fill: parent

        MenuButton {
            id: splitModel
            menuButtonText: "Auto place models"
            buttonAction.onClicked: Gence.Actions.arrange.trigger()
            helpText: "Arranges the loaded files in the printer buildplate automatically "+
                      "to prevent them from intersecting. Arranges object bases on the " +
                      "buildplate. The models will be placed at the buildplates corner by default."
        }

        MenuButton {
            id: addModel
            menuButtonText: "Align face to bed"
            buttonAction.onClicked: Cura.Actions.toggleOrientFaceToPlane()
            helpText: "Choose the wall that will become the basis of the print."
        }

        MenuButton {
            id: enableAdvancedOptions
            visible: !Globals.multiextruderPrint
            menuButtonText: "Multimaterial options"
            buttonAction.onClicked: warningDialog.open()
            helpText: "Enable advanced multimaterial options.\n\nRead manual to further details."
        }

        MenuButton {
            id: alignSelectedModels
            deactivate: !UM.Selection.hasSelection || UM.Selection.hasSingleSelection
            visible: Globals.multiextruderPrint
            menuButtonText: "Align selected models"
            buttonAction.onClicked: Printer.alignSelected()
            helpText: "Align selected models with the same origin point for the " +
            "advanced multimaterial print.\n\nRead manual to further details."
        }
    }

    MessageDialog {
        id: warningDialog
        title: "Multimaterial options warning"
        icon: StandardIcon.Warning
        text: "These options were prepared for advanced users for multimaterial print " +
              "beyond standard applications of the 3D printers.\n\n" +
              "Please read manual for further details. You are using these options AT YOUR OWN RISK, without any warranties. " +
              "Some modifications may damage the print, the printing module or the printer - 3DGence takes no resposibility " +
              "for damage caused by inappropriate use of these settings."
        standardButtons: StandardButton.Cancel | StandardButton.Ok
        onAccepted: Globals.multiextruderPrint = true
    }
}
