import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../../singletons"
import "../../tweaks"
import "../../list"
import ".."

import Gence 1.0 as Gence
import Settings 1.0 as Settings


Item {
    property alias layoutHeight: layout.height
    property alias layout: layout
    property alias list1Height: list1.listHeight
    property alias list2Height: list2.listHeight
    property alias loadCustomButtonHeight: loadCustom.height
    property alias editCustomButtonHeight: editCustom.height
    property alias list1: list1
    property alias list2: list2

    ColumnLayout {
        anchors.fill: parent
        id: layout

        LabeledRetractableList {
            id: list1
            labelText: "MoNo:"
            listModel: Gence.ModelsHolder.nozzles
            onSelectedChanged:
            {
                if(selected !== null)
                {
                    if(isNaN(selected.text)) {
                        labelText="Module:"
                    } else {
                        labelText="Nozzle:"
                    }
                    Settings.Notifications.nozzle = selected.text
                }
            }

            helpText: "Select the module type or the nozzle size to the printer used."
        }

        LabeledRetractableList {
            id: list2
            labelText: "Material:"
            listModel: Gence.ModelsHolder.materials
            updateButtonOption: true
            onSelectedChanged:
            {
                if(selected !== null) {
                    Settings.Notifications.material = selected.text
                }
            }
            helpText: "Select the material to be used for the model."
        }

        MenuButton {
            id: loadCustom
            menuButtonText: "Load custom profile"
            buttonClickedAction.onTriggered:
            {
                var result = Gence.CustomSettingsActions.loadProfileDialog()

                if(!result) {
                    Printer.displayStatus("Incorrect custom profile.")
                }
            }
            helpText: "Load the custom profile without editing it."
        }

        MenuButton {
            id: editCustom
            menuButtonText: "Edit custom profile (advanced)"
            buttonClickedAction.onTriggered:
            {
                Globals.modalWindow.customSettingsWarning()
            }

            property string shortcutString: Qt.platform.os === "osx" ? "⌘+E" : "Ctrl+E"
            helpText: "Load and edit the custom profile or edit previously " +
            "loaded custom profile.\n\nKeyboard shortcut - " + shortcutString
        }

        Spacer {}
    }
}
