import QtQuick 2.4

import UM 1.2 as UM
import Gence 1.0 as Gence

import "../../singletons"

SplitButtonContentForm {
    property bool excludeFlag: false
    property bool advancedMode: cutTypeString == "Advanced"

    Connections {
        id: sliderToSpinBox
        target: slider
        onValueChanged: {
            if(!excludeFlag) {
                excludeFlag = true
                spinBox.valueText = slider.value
                excludeFlag = false
            }
        }
    }

    Connections {
        id: spinBoxToSlider
        target: spinBox.onValueChanged
        onTriggered: {
            if(!excludeFlag) {
                excludeFlag = true
                slider.value = Math.round(spinBox.value)
                excludeFlag = false
            }
            UM.ActiveTool.setProperty("Value", spinBox.value)
        }
    }


/////////////
    Connections {
        target: edgeSpinBox.onValueChanged
        onTriggered: {
            if(edgeSpinBox.validValue){
                UM.ActiveTool.setProperty("Edge", edgeSpinBox.value)
            }
        }
    }

    Connections {
        target: angleSpinBox.onValueChanged
        onTriggered: {
            UM.ActiveTool.setProperty("Angle", angleSpinBox.value)
        }
    }

    Connections {
        target: offsetSpinBox.onValueChanged
        onTriggered: {
            UM.ActiveTool.setProperty("Offset", offsetSpinBox.value)
        }
    }

    Connections {
        target: lengthSpinBox.onValueChanged
        onTriggered: {
            UM.ActiveTool.setProperty("Length", lengthSpinBox.value)
        }
    }

    Connections {
        target: invertCheckBox.checkBox
        onClicked: {
            UM.ActiveTool.setProperty("Invert", invertCheckBox.value)
        }
    }

 ///////////////

    Connections {
        target: cutTypeButton.buttonClickedAction
        onTriggered: {
            var cutType = ""

            switch(cutTypeString){
                case "Advanced":
                    cutType = "Simple"
                    objectHelpText = Help.simpleSplit
                    objectHelpImageSource = Help.simpleSplitImg
                    setSimpleSplitButtonsStatus(true)
                    axisString = "X"
                    UM.ActiveTool.setProperty("Axis", "X")
                    UM.ActiveTool.setProperty("Value", slider.value)
                    break;
                case "Simple":
                    cutType = "Divide";
                    objectHelpText = Help.divideSplit
                    objectHelpImageSource = ""
                    setSimpleSplitButtonsStatus(false);
                    axisString = "X, Y, Z";
                    break;
                case "Divide":
                    cutType = "Advanced";
                    objectHelpText = Help.advancedSplit
                    objectHelpImageSource = Help.advancedSplitImg
                    setSimpleSplitButtonsStatus(true);
                    axisString = "X"
                    UM.ActiveTool.setProperty("Axis", "X")
                    UM.ActiveTool.setProperty("Value", slider.value)

                    // Default values for advanced split are set here:
                    edgeSpinBox.valueText = "1.5"
                    angleSpinBox.valueText = "75"
                    offsetSpinBox.valueText = "0.3"
                    lengthSpinBox.valueText = "3"
                    invertCheckBox.value = false

                    UM.ActiveTool.setProperty("Edge", 1)
                    UM.ActiveTool.setProperty("Angle", 45)
                    UM.ActiveTool.setProperty("Offset", 0)
                    UM.ActiveTool.setProperty("Length", 5)
                    UM.ActiveTool.setProperty("Invert", false)

                    break;
                default: //Just in case
                    cutType = "Divide";
                    setSimpleSplitButtonsStatus(false);
                    axisString = "X, Y, Z";
            }

            UM.ActiveTool.setProperty("CutType", cutType)
            cutTypeString = cutType
        }
    }

    Connections{
        target: Gence.Actions.showSplitTool
        onTriggered:{
            cutTypeString = "Simple";
            axisString = "X";
            objectHelpText = Help.simpleSplit
            objectHelpImageSource = Help.simpleSplitImg
            setSimpleSplitButtonsStatus(true);

            var boundaries = Printer.getSplitMinMax
            var min = Math.floor(boundaries[0])
            var max = Math.ceil(boundaries[1])


            setMin(min)
            setMax(max)

            slider.value = (min + max) / 2
        }
    }

    Connections {
        target: axisButton.buttonClickedAction
        onTriggered: {
            var axis = ""

            switch(axisString){
                case "X": axis = "Y"; break;
                case "Y": axis = "Z"; break;
                case "Z": axis = "X"; break;
                default: axis = "X" //just in case
            }

            UM.ActiveTool.setProperty("Axis", axis)
            axisString = axis
        }
    }

    Connections {
        target: Gence.Actions.splitToolLimitsChanged
        onTriggered: {
           //UM.ActiveTool.forceUpdate()
           var max = Math.ceil(UM.ActiveTool.properties.getValue("CutMax"))
           var min = Math.floor(UM.ActiveTool.properties.getValue("CutMin"))

            setMin(min)
            setMax(max)

            slider.value = (min + max) / 2
        }
    }

    Connections
    {
        target: Gence.Notifications
        onSplittingState: splittingState(state)
    }

    function splittingState(state){
        switch(state){
            case "Splitting":
                Gence.Actions.deactivate_right_menu.trigger()
                Gence.Actions.split_deactivate_left_menu.trigger()
                enableUI = false
                break
            case "Success":
                Gence.Actions.activate_right_menu.trigger()
                Gence.Actions.split_succ_activate_left_menu.trigger()
                enableUI = true
                break
            case "Fail":
                Gence.Actions.activate_right_menu.trigger()
                Gence.Actions.split_fail_activate_left_menu.trigger()
                enableUI = true
                break
        }
    }

    function setSimpleSplitButtonsStatus(isActive) {
         spinBox.deactivate = !isActive
         axisButton.deactivate = !isActive
         slider.deactivate = !isActive
    }

    function setMin(value){
        spinBox.minValue = value
    }

    function setMax(value){
        spinBox.maxValue = value
    }
}
