import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../singletons"
import "../tweaks"

Item {
    property alias cameraCube: cameraCube

    property alias topSide: topSide
    property alias rightSide: rightSide
    property alias leftSide: leftSide

    property alias topSideMouseArea: topSideMouseArea
    property alias leftSideMouseArea: leftSideMouseArea
    property alias rightSideMouseArea: rightSideMouseArea
    property alias dot1MouseArea:  dot1.mouseArea
    property alias dot2MouseArea:  dot2.mouseArea
    property alias dot3MouseArea:  dot3.mouseArea
    property alias dot4MouseArea:  dot4.mouseArea
    property alias dot5MouseArea:  dot5.mouseArea
    property alias dot6MouseArea:  dot6.mouseArea
    property alias dot7MouseArea:  dot7.mouseArea

    Layout.fillWidth: true
    Layout.fillHeight: true
    
    Item {
        id: cameraCube

        anchors.fill: parent
        Rectangle {
            id: topSide
            width: parent.width * 0.3
            height: parent.width * 0.3
            border.width: 1
            antialiasing: true

            color: topSideActive ? Colors.cubeActiveSide : Colors.cubeInactiveSide

            MouseArea {
                id: topSideMouseArea
                anchors.fill: parent
            }
        }

        Rectangle {
            id: rightSide

            x: -1
            y: (topSide.width*Math.sqrt(2)*Math.tan(Math.PI/6))
            width: parent.width * 0.3
            height: parent.width * 0.3
            border.width: 1
            antialiasing: true

            color: rightSideActive ? Colors.cubeActiveSide : Colors.cubeInactiveSide

            MouseArea {
                id: rightSideMouseArea
                anchors.fill: parent
            }
        }

        Rectangle {
            id: leftSide

            x: -(topSide.width*Math.sqrt(2))/2 + 1
            y: (topSide.width*Math.sqrt(2)*Math.tan(Math.PI/6))/2
            width: parent.width * 0.3
            height: parent.width * 0.3
            border.width: 1
            antialiasing: true

            color: leftSideActive ? Colors.cubeActiveSide : Colors.cubeInactiveSide

            MouseArea {
                id: leftSideMouseArea
                anchors.fill: parent
            }
        }

        CubeDot {
            id: dot1

            x: -topSide.width * 0.07
            y: -topSide.width * 0.04

            activityIndicator: dot1Active
        }

        CubeDot {
            id: dot2

            x: -topSide.width * 0.75
            y: topSide.width * 0.35

            activityIndicator: dot2Active
        }

        CubeDot {
            id: dot3

            x: topSide.width * 0.6
            y: topSide.width * 0.35

            activityIndicator: dot3Active
        }

        CubeDot {
            id: dot5

            x: -topSide.width * 0.75
            y: topSide.width * 1.1

            activityIndicator: dot5Active
        }

        CubeDot {
            id: dot4

            x: -topSide.width * 0.07
            y: topSide.width * 0.75

            activityIndicator: dot4Active
        }

        CubeDot {
            id: dot6

            x: topSide.width * 0.6
            y: topSide.width * 1.1

            activityIndicator: dot6Active
        }

        CubeDot {
            id: dot7

            x: -topSide.width * 0.07
            y: topSide.width * 1.5

            activityIndicator: dot7Active
        }

        Image {
            x: topSide.width * 0.36
            y: topSide.width * 1.4
            height: topSide.width * 0.2
            rotation: 15
            mipmap: true
            antialiasing: true

            fillMode: Image.PreserveAspectFit
            source: "../images/corner.svg"
        }
    }

    ButtonTweaks {
        hoverIndicatorEnabled: false
        helpText: "Click wall or vertex on the cube to select a predefined point of view"
    }
}
