import QtQuick 2.4

import "../singletons"

RetractableListForm {

    // simple this reference
    property var this_component : this;

    // save these values externally in order to avoid problems with temporary
    // values that can appear during animations/hiding/expanding etc.
    property int pos_x : 0
    property int pos_y : 0

    Connections {
        target : combo_box_manager
        onHideComboBoxes : {
            if (this_component.visible) {
                // on appearance - store a screen position, it will be
                // used later in a hide combo box event handler (rised by
                // an outside user action)
                var pos = this_component.mapToItem (null, 0, 0);
                pos_x   = pos.x
                pos_y   = pos.y

                // make exception if we are trying to hide active combo box -
                // it will be hidden anyway by 'RetractableList' mechanisms
                var w        = this_component.width;
                var h        = this_component.height;
                var not_in_x = mx < pos_x || mx > pos_x + w;
                var not_in_y = my < pos_y || my > pos_y + h;
                
                if (not_in_x || not_in_y) {
                    closeList ();
                }
            }
        }
    }

    Connections {
        target    : mouseArea
        onClicked : changeListState ()
    }


    // user clicked a list element
    onListElementClicked : {
        closeList ();
    }


    function onResize () {
        if (listOpened) {
            menuHeightMod = background.height * heightMod;
        }
    }


    function changeListState () {
        if(listOpened){
            closeList ();
        } else {
            openList ();
        }
    }


    function openList () {
        listOpened       = true;
        background.color = Colors.listHighlightedBackground;
        text.color       = Colors.listHighlightedText;

        // inform the manager about an opening - will tell
        // other combo boxes to close themselves
        combo_box_manager.shown ();
    }

    function closeList () {
        listOpened       = false;
        background.color = Colors.listBackground;
        text.color       = Colors.listText;
    }

    function changeCurrentText(value){
        text.text = value
    }

    function restoreCurrentValue(){
        // text.text = list.currentItem ? list.currentItem.text : "empty"
    }


    function disableList(){
        listEnabled = false
    }

    function enableList(){
        listEnabled = true
    }
}
