import QtQuick 2.7
import QtQuick.Layouts 1.1
import QtQuick.Controls 2.0

import "../singletons"

Item {
    property bool listOpened: false
    property bool scrollable: false
    property int listHeight: list.height * listOpened + background.height
    property int listWidth: background.width
    property alias list: list
    property alias selected: list.currentItem
    property alias currentIndex: list.currentIndex
    property alias listModel: list.model
    property alias background: background
    property alias text: textField
    property alias mouseArea: mouseArea
    property alias listEnabled: mouseArea.visible
    property alias defaultListText: textField.text

    // rised on list element clicked
    signal listElementClicked
    Layout.fillWidth: true
    Layout.preferredHeight: listHeight

    // main bar - defines most dimensions in this component!
    Rectangle {
        id: background
        width: parent.width
        height: textField.height * 2
        color: Colors.listBackground
        border.color: Colors.listBorder
        border.width: 1

        Text {
            id: textField
            width: parent.width * 0.86
            anchors.verticalCenter: parent.verticalCenter
            anchors.left: parent.left
            anchors.leftMargin: parent.width * 0.04
            color: Colors.listText
            font.pixelSize: Globals.fontPixelSize
            font.family: Fonts.boxed
            // text                    : list.currentItem ? list.currentItem.text : "empty"
            elide: Text.ElideMiddle
        }

        Image {
            id: expandIconGrey
            visible: !listOpened
            anchors.verticalCenter: parent.verticalCenter
            anchors.right: parent.right
            anchors.rightMargin: parent.width * 0.04
            width: parent.height * 0.35
            mipmap: true
            fillMode: Image.PreserveAspectFit
            source: "../images/unfold.svg"
        }

        Image {
            id: expandIconWhite
            visible: listOpened
            anchors.verticalCenter: parent.verticalCenter
            anchors.right: parent.right
            anchors.rightMargin: parent.width * 0.04
            width: parent.height * 0.35
            mipmap: true
            fillMode: Image.PreserveAspectFit
            source: "../images/unfold_white.svg"
        }
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        hoverEnabled: false
        visible: true
    }

    Rectangle {
        id: deactivateIndicator
        anchors.fill: background
        visible: !mouseArea.visible
        color: Colors.buttonDeactivator
    }

    // list of all buttons
    ListView {
        id: list
        visible: listOpened
        width: parent.width
        anchors.top: background.bottom
        height: (scrollable ? Math.min(count, 8) : count) * background.height
        boundsBehavior: Flickable.StopAtBounds
        clip: true
        model: null

        ScrollBar.vertical: scrollable ? scroll : null    

        onCountChanged: {
            currentIndex = 0
        }

        onCurrentItemChanged: {
            if (currentItem != null)
                textField.text = currentItem.text
        }

        // displays list position
        delegate: RetractableListElement {
            // property alias entry: entry
            scrollableElement: scrollable
            id: entry
            text: name
            updateOption: updateButtonOption
            onElementClicked: listElementClicked()
        }
    }

    ScrollBar {
        id: scroll
        parent: list
        anchors.top: background.bottom
        anchors.right: list.right
        anchors.rightMargin: 1
        visible: list.contentHeight > list.height && list.visible && scrollable ? true : false
        contentItem: Rectangle {
            implicitWidth: list.width * 0.02
            color: Colors.customListScrollBarHandle
        }
        background: Rectangle {
            anchors.fill: parent
            anchors.topMargin: 1
            anchors.bottomMargin: 1
            color: Colors.customListBackground
        }
    }
}
