// Copyright (c) 2015 Ultimaker B.V.
// Uranium is released under the terms of the AGPLv3 or higher.

import QtQuick 2.7
import QtQuick.Controls 1.1
import QtQuick.Controls.Styles 1.3

import UM 1.1 as UM

import "../../singletons"

SettingItem
{
    id: base

    contents: ComboBox
    {
        id: control

        model: definition.options
        textRole: "value"

        anchors.fill: parent

        MouseArea
        {
            anchors.fill: parent;
            acceptedButtons: Qt.NoButton;
            onWheel: wheel.accepted = true;
        }


        style: ComboBoxStyle
        {

            font.family: Fonts.boxed
            font.pixelSize: Globals.fontPixelSize
            textColor: Colors.customListText
            selectionColor: Colors.customListSelectedBackground
            selectedTextColor: Colors.customListSelectedText
            padding.left: 0
            padding.right: 0

            background: Rectangle
            {
                color: enabled ? Colors.enabledBackground : Colors.disabledBackground
                border.width: 1
                border.color: Colors.customSettingsEditBorder
            }
            
            label: Item
            {   
                Label
                {
                    anchors.left: parent.left
                    anchors.leftMargin: Globals.margin
                    anchors.right: downArrow.left
                    anchors.rightMargin: Globals.margin
                    anchors.verticalCenter: parent.verticalCenter

                    text: control.currentText;
                    font.family: Fonts.boxed
                    font.pixelSize: Globals.fontPixelSize
                    color: Colors.customListText

                    elide: Text.ElideRight;
                    verticalAlignment: Text.AlignVCenter;
                }

                Image {
                    id: downArrow
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.right: parent.right
                    anchors.rightMargin: Globals.margin
                    width: parent.height * 0.6
                    mipmap: true
                    fillMode: Image.PreserveAspectFit
                    source: "../../images/unfold.svg"
                }
            }
        }

        onActivated: {
            forceActiveFocus();
            var key = ""
            if (basicTabActive){
                key = definition.options.get(index).key
                currentIndex = index
            } else {
                key = definition.options[index].key
            }
            propertyProvider.setPropertyValue("value", key) 
        }

        Binding
        {
            target: control
            property: "currentIndex"
            value:
            {
                // FIXME this needs to go away once 'resolve' is combined with 'value' in our data model.
                var value;
                if ((base.resolve != "None") && (base.stackLevel != 0) && (base.stackLevel != 1)) {
                    // We have a resolve function. Indicates that the setting is not settable per extruder and that
                    // we have to choose between the resolved value (default) and the global value
                    // (if user has explicitly set this).
                    value = base.resolve;
                } else {
                    value = propertyProvider.properties.value;
                }

                if (basicTabActive){
                    for(var i = 0; i < control.model.count; ++i) {
                        if(control.model.get(i).key == value) {
                            return i;
                        }
                    }
                } else {
                    for(var i = 0; i < control.model.length; ++i) {
                        if(control.model[i].key == value) {
                            return i;
                        }
                    }
                }

                return -1;
            }
        }
    }
}
