import QtQuick 2.4
import QtQuick.Controls 1.4

import Gence 1.0 as Gence
import UM 1.3 as UM
import Settings 1.0 as Settings

import "../../JS/toolbox.js" as Toolbox

CustomSpinBoxForm
{
    property alias onValueChanged: valueWasChangedByInputText
    Action
    {
        id: valueWasChangedByInputText
    }

    //Mark that there were some changes in text
    Connections
    {
        target: textField
        onTextChanged:
        {
            if(instant && /\d/.test(valueText)) {
                checkNewInput()
            } else {
                wasTextChanged = true
            }
        }
    }

    //If text field is loosing focus then check if something was written, if text field gain focus then save last proper value
    Connections
    {
        target: textField
        onFocusChanged:
        {
            if(!textField.focus)
            {
                validateAndCheckNewInput()
            }
            // else
            // {
            //     UM.Selection.isDeaf = true;
            // }
        }
    }

    //If user has pressed accept button, check if there were any changes
    Connections
    {
        target:textField
        onAccepted:
        {
            validateAndCheckNewInput()
        }
    }

    function validateAndCheckNewInput(){
        var currentValue = parseFloat(valueText)
        if(!isGivenValueValid(currentValue)){
            if(currentValue > maxValue){
                valueText = maxValue.toString()
            } else if (currentValue < minValue){
                valueText = minValue.toString()
            }
        } else if(roundToStep && (currentValue % singleStep)){
            roundToStepFun(currentValue)
        }
        checkNewInput()
    }

    function checkNewInput()
    {
        wasTextChanged = false

        if(valueText.length === 0){
            textFieldReset()
        }

        // Ugly hack - we just want to trigger change on value
        var new_value = parseFloat(valueText).toFixed(precision)
        if(new_value != value){
            value = new_value
            valueModified()
        }
        
        focus = false
    }

    function textFieldReset()
    {
        valueText = Math.max(0, minValue).toString()
    }

    function roundToStepFun(value)
    {
        if(value % singleStep > singleStep / 2){
            value = Math.ceil(value / singleStep) * singleStep
        } else {
            value = Math.floor(value / singleStep) * singleStep
        }

        valueText = value.toString()
    }

    // ==============================> standalone spinbox functions and connections <==============================
    function valueModified(){
        if(standalone){
            var trailingDot = (valueText.slice(-1) == '.')
            var value = getNumValue()
            if(value.status) {
                validValue = true
                if(curaParam != "") {
                    Settings.Notifications.set_cura_param(curaParam, parseFloat(value.value))
                }
                var suffix = ''
                if(trailingDot){
                    suffix = '.'
                }
                valueText = value.value.toString() + suffix
            } else {
                validValue = false
            }
        }
        valueWasChangedByInputText.trigger()
    }

    function getNumValue() {
        if(standalone){
            var value = parseFloat(valueText)
            var valueToReturn = 0
            var statusToReturn = false

            if(isGivenValueValid(value)) {
                statusToReturn = true
                valueToReturn = Math.round(value*Math.pow(10, precision)) / Math.pow(10, precision)
            }

            return {
                status: statusToReturn,
                value: valueToReturn
            }
        }
    }

    function initCuraParam() {
        if(curaParam != ""){
            var result = Settings.Notifications.init_cura_param(curaParam)
            spinBoxAlias.valueText = Math.round(parseFloat(result)*Math.pow(10, precision)) / Math.pow(10, precision)
        }
    }

    function resetCuraParam() {
        Settings.Notifications.set_cura_param(curaParam, "None")
    }

    function isGivenValueValid(value){
        if(value >= minValue && value <= maxValue){
            return true
        }

        return false
    }

    function isRoundingNeeded(value) {
        if (roundToStep && (value * Math.pow(10, precision)) % (singleStep * Math.pow(10, precision))) {
            return true
        }

        return false
    }

    //TODO wyrzucić if(standalone) i dodać do samego connection pole enabled: standalone!
    Connections
    {
        target: minusButtonAction
        onClicked:
        {
            if(standalone){
                var currentValue = parseFloat(valueText)
                if(isGivenValueValid(currentValue)){
                    if(isRoundingNeeded()){
                        currentValue = Math.floor(currentValue / singleStep) * singleStep
                    } else {
                        currentValue = Math.max(minValue, currentValue - singleStep)
                    }
                } else {
                    currentValue = minValue
                }
                currentValue = Toolbox.round(currentValue, precision)
                valueText = currentValue.toString()
            }
        }
    }

    Connections
    {
        target: plusButtonAction
        onClicked:
        {
            if(standalone){
                var currentValue = parseFloat(valueText)
                if(isGivenValueValid(currentValue)){
                    if(isRoundingNeeded()){
                        currentValue = Math.ceil(currentValue / singleStep) * singleStep
                    } else {
                        currentValue = Math.min(maxValue, currentValue + singleStep)
                    }
                } else {
                    currentValue = maxValue
                }
                currentValue = Toolbox.round(currentValue, precision)
                valueText = currentValue.toString()
            }
        }
    }
}
