import QtQuick 2.2

ButtonTweaksForm {
    property bool helpEnabled: true

    anchors.fill: parent

    onHelpEnabledChanged: {
        if(!helpEnabled){
            popupTimer.stop()
        }
    }

    Connections {
        target: helpMouseArea
        onEntered: onMouseEntered()
        onExited: onMouseExit()
        onPositionChanged: onMouseMoved()
        onClicked: onMouseClicked(mouse.button)
        onPressed: onMousePressed(mouse)
    }


    Timer {
        id: popupTimer
        interval: 500
        running: false
        repeat: false
        onTriggered: openHelpPopup()
    }

    function openHelpPopup() {
        var position = getPopupPosition()
        if(helpMouseArea.containsMouse){
            helpPopup.x = position.x
            helpPopup.y = position.y
            helpPopup.open()
        }
    }

    function onMouseClicked(button) {
        if(button === Qt.RightButton && helpEnabled){
            openHelpPopup()
        }
    }

    function onMousePressed(mouse) {
        if(mouse.button === Qt.LeftButton){
            mouse.accepted = false
            if(helpEnabled){
                popupTimer.stop()
            }            
        }
    }

    function onMouseMoved() {
        helpPopup.close()
        if(helpEnabled && !(helpMouseArea.pressedButtons & Qt.LeftButton)) popupTimer.restart()
    }

    function onMouseEntered() {
        onEnteredAnimation.start()
        if(helpEnabled) popupTimer.restart()
    }

    function onMouseExit() {
        onExitedAnimation.start()
        if(helpEnabled) {
            helpPopup.close()
            popupTimer.stop()
        }
    }

    function getPopupPosition() {
        /*
          Calculates coordinates to spawn popup, center is mouse position:

           3 | 4
          ---+---
           2 | 1

        */

        var parentCoordinates = parent.mapToItem(mainForm, 0, 0)
        var fitsWidth = parentCoordinates.x + helpMouseArea.mouseX + helpPopup.width + 1 <= mainForm.width
        var fitsHeight = parentCoordinates.y + helpMouseArea.mouseY + helpPopup.height + 1 <= mainForm.height

        // 1
        if(fitsWidth && fitsHeight){
            return {x: helpMouseArea.mouseX + 1, y: helpMouseArea.mouseY}
        }
        // 2
        if(!fitsWidth && fitsHeight){
            return {x: helpMouseArea.mouseX - helpPopup.width - 1, y: helpMouseArea.mouseY}
        }
        // 3
        if(!fitsWidth && !fitsHeight){
            return {x: helpMouseArea.mouseX - helpPopup.width - 1, y: helpMouseArea.mouseY - helpPopup.height}
        }
        //4
        if(fitsWidth && !fitsHeight){
            return {x: helpMouseArea.mouseX + 1, y: helpMouseArea.mouseY - helpPopup.height}
        }
    }

    PropertyAnimation on hoverIndicatorOpacity {
        id: onEnteredAnimation
        easing.type: Easing.OutExpo
        duration: 50
        to: 0.4
        running: false
    }

    PropertyAnimation on hoverIndicatorOpacity {
        id: onExitedAnimation
        duration: 50
        to: 0.0
        running: false
    }
}
